//
//  ManhattanView.swift
//  Manhattan
//
//  Created by Mark Meretzky on 10/28/18.
//  Copyright © 2018 New York University School of Professional Studies. All rights reserved.
//

import UIKit;
import GLKit;        //needed for GLKMathDegreesToRadians

class ManhattanView: UIView {
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)!;
        backgroundColor = UIColor.white; //Unnecessary: it's the default.
    }

    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation
    
    override func draw(_ rect: CGRect) {
        // Drawing code
        
        let points: [CGPoint] = [    //an array of CGPoint structures
            CGPoint(x: -73.971548, y: 40.729218),    //East River at East 17th Street
            CGPoint(x: -73.974595, y: 40.735519),    //24
            CGPoint(x: -73.971806, y: 40.742998),    //34
            CGPoint(x: -73.96215,  y: 40.754767),    //53
            CGPoint(x: -73.954296, y: 40.762146),    //65
            CGPoint(x: -73.946185, y: 40.771474),    //81
            CGPoint(x: -73.942022, y: 40.776154),    //89
            CGPoint(x: -73.942022, y: 40.776154),    //96
            CGPoint(x: -73.93816,  y: 40.787008),    //103
            CGPoint(x: -73.929534, y: 40.795326),    //118
            CGPoint(x: -73.929062, y: 40.800946),    //125
            CGPoint(x: -73.934212, y: 40.808775),    //Harlem River at 132nd Street
            CGPoint(x: -73.933868, y: 40.817772),    //143
            CGPoint(x: -73.935113, y: 40.83547),     //163
            CGPoint(x: -73.922195, y: 40.855857),    //Dyckman Street
            CGPoint(x: -73.91078,  y: 40.869878),    //218
            CGPoint(x: -73.911767, y: 40.873416),    //Broadway Bridge
            CGPoint(x: -73.922968, y: 40.877018),    //Henry Hudson Parkway Bridge
            CGPoint(x: -73.926916, y: 40.877082),    //Hudson River
            CGPoint(x: -73.933096, y: 40.867379),    //Riverside Drive
            CGPoint(x: -73.943224, y: 40.852417),    //Hudson River at West 181st Street
            CGPoint(x: -73.946786, y: 40.850339),    //George Washingon Bridge
            CGPoint(x: -73.946786, y: 40.850339),    //168
            CGPoint(x: -73.95052,  y: 40.834626),    //155
            CGPoint(x: -73.955026, y: 40.827417),    //144 sewage treatment plant
            CGPoint(x: -73.956399, y: 40.828034),    //144
            CGPoint(x: -73.959446, y: 40.82365),     //137
            CGPoint(x: -73.957601, y: 40.822676),    //137
            CGPoint(x: -73.994765, y: 40.771669),    //57
            CGPoint(x: -73.995152, y: 40.769524),    //54
            CGPoint(x: -73.999872, y: 40.763316),    //44
            CGPoint(x: -74.001718, y: 40.762276),    //42
            CGPoint(x: -74.007726, y: 40.754052),    //29
            CGPoint(x: -74.009442, y: 40.749825),    //23
            CGPoint(x: -74.00794,  y: 40.748362),    //21
            CGPoint(x: -74.009228, y: 40.740754),    //Meatpacking District
            CGPoint(x: -74.010344, y: 40.739258),    //Gansevoort Street
            CGPoint(x: -74.011545, y: 40.726218),    //Holland Tunnel
            CGPoint(x: -74.013176, y: 40.718315),    //Battery Park City
            CGPoint(x: -74.016609, y: 40.718737),    //Battery Park City
            CGPoint(x: -74.019227, y: 40.706539),    //South Cove
            CGPoint(x: -74.014893, y: 40.70078),     //Battery Park
            CGPoint(x: -74.009314, y: 40.701919),    //Heliport
            CGPoint(x: -73.997984, y: 40.708523),    //north of Brooklyn Bridge
            CGPoint(x: -73.977985, y: 40.710475),    //Corlears Hook Park
            CGPoint(x: -73.976011, y: 40.712752),    //Grand Street
            CGPoint(x: -73.972964, y: 40.720819)
        ];
        
        //Latitude and longitude in degrees of the heart of Manhattan,
        //probably somewhere around the Angel of the Waters in Central Park.
        let latitude: CGFloat =    40.79;    //north of equator is positive
        let longitude: CGFloat =  -73.96;    //west of Greenwich is negative
        
        //How many pixels per degree of latitude (approx. 60 miles).
        //The screen will cover 1/4 of a degree of latitude vertically.
        let scale: CGFloat = 4 * bounds.height;
        
        let c: CGContext = UIGraphicsGetCurrentContext()!;
        c.beginPath();    //unnecesary here: the path is already empty
        
        //Three transformations:

        //Transformation 1: move the origin (0, 0) to the center of the View.
        c.translateBy(x: bounds.width / 2, y: bounds.height / 2);
        
        //Transformation 2: magnify Manhattan to make it big enough to see.
        //Also make the Y axis point upwards so that north is up.
        let radians: CGFloat = CGFloat(GLKMathDegreesToRadians(Float(latitude)));
        let cosineOfLatitude: CGFloat = cos(radians);
        c.scaleBy(x: scale * cosineOfLatitude, y: -scale);
        
        //Transformation 3: move the camera from the South Atlantic Ocean
        //left and up to Manhattan.  More precisely, move Manhattan right
        //and down to the South Atlantic.  That's why there are negative signs.
        c.translateBy(x: -longitude, y: -latitude);
        
        c.move(to: points.first!);
        for point in points.dropFirst() {   //all the CGPoints except the first one
            c.addLine(to: point);
        }
        c.closePath();   //Add a line back to the starting point.
    
        c.setFillColor(red: 0, green: 0, blue: 1, alpha: 1);    //blue, opaque
        c.fillPath();
    }

}
